# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['ConfigEntryArgs', 'ConfigEntry']

@pulumi.input_type
class ConfigEntryArgs:
    def __init__(__self__, *,
                 kind: pulumi.Input[str],
                 config_json: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 partition: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a ConfigEntry resource.
        :param pulumi.Input[str] kind: The kind of configuration entry to register.
        :param pulumi.Input[str] config_json: An arbitrary map of configuration values.
        :param pulumi.Input[str] name: The name of the configuration entry being registered.
        :param pulumi.Input[str] namespace: The namespace to create the config entry within.
        :param pulumi.Input[str] partition: The partition the config entry is associated with.
        """
        pulumi.set(__self__, "kind", kind)
        if config_json is not None:
            pulumi.set(__self__, "config_json", config_json)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if partition is not None:
            pulumi.set(__self__, "partition", partition)

    @property
    @pulumi.getter
    def kind(self) -> pulumi.Input[str]:
        """
        The kind of configuration entry to register.
        """
        return pulumi.get(self, "kind")

    @kind.setter
    def kind(self, value: pulumi.Input[str]):
        pulumi.set(self, "kind", value)

    @property
    @pulumi.getter(name="configJson")
    def config_json(self) -> Optional[pulumi.Input[str]]:
        """
        An arbitrary map of configuration values.
        """
        return pulumi.get(self, "config_json")

    @config_json.setter
    def config_json(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "config_json", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the configuration entry being registered.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        The namespace to create the config entry within.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter
    def partition(self) -> Optional[pulumi.Input[str]]:
        """
        The partition the config entry is associated with.
        """
        return pulumi.get(self, "partition")

    @partition.setter
    def partition(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "partition", value)


@pulumi.input_type
class _ConfigEntryState:
    def __init__(__self__, *,
                 config_json: Optional[pulumi.Input[str]] = None,
                 kind: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 partition: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering ConfigEntry resources.
        :param pulumi.Input[str] config_json: An arbitrary map of configuration values.
        :param pulumi.Input[str] kind: The kind of configuration entry to register.
        :param pulumi.Input[str] name: The name of the configuration entry being registered.
        :param pulumi.Input[str] namespace: The namespace to create the config entry within.
        :param pulumi.Input[str] partition: The partition the config entry is associated with.
        """
        if config_json is not None:
            pulumi.set(__self__, "config_json", config_json)
        if kind is not None:
            pulumi.set(__self__, "kind", kind)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if partition is not None:
            pulumi.set(__self__, "partition", partition)

    @property
    @pulumi.getter(name="configJson")
    def config_json(self) -> Optional[pulumi.Input[str]]:
        """
        An arbitrary map of configuration values.
        """
        return pulumi.get(self, "config_json")

    @config_json.setter
    def config_json(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "config_json", value)

    @property
    @pulumi.getter
    def kind(self) -> Optional[pulumi.Input[str]]:
        """
        The kind of configuration entry to register.
        """
        return pulumi.get(self, "kind")

    @kind.setter
    def kind(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "kind", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the configuration entry being registered.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        The namespace to create the config entry within.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter
    def partition(self) -> Optional[pulumi.Input[str]]:
        """
        The partition the config entry is associated with.
        """
        return pulumi.get(self, "partition")

    @partition.setter
    def partition(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "partition", value)


class ConfigEntry(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 config_json: Optional[pulumi.Input[str]] = None,
                 kind: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 partition: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        ## Example Usage

        ```python
        import pulumi
        import json
        import pulumi_consul as consul

        proxy_defaults = consul.ConfigEntry("proxy_defaults",
            kind="proxy-defaults",
            name="global",
            config_json=json.dumps({
                "Config": {
                    "local_connect_timeout_ms": 1000,
                    "handshake_timeout_ms": 10000,
                },
            }))
        web = consul.ConfigEntry("web",
            name="web",
            kind="service-defaults",
            config_json=json.dumps({
                "Protocol": "http",
            }))
        admin = consul.ConfigEntry("admin",
            name="admin",
            kind="service-defaults",
            config_json=json.dumps({
                "Protocol": "http",
            }))
        service_resolver = consul.ConfigEntry("service_resolver",
            kind="service-resolver",
            name=web.name,
            config_json=json.dumps({
                "DefaultSubset": "v1",
                "Subsets": {
                    "v1": {
                        "Filter": "Service.Meta.version == v1",
                    },
                    "v2": {
                        "Filter": "Service.Meta.version == v2",
                    },
                },
            }))
        service_splitter = consul.ConfigEntry("service_splitter",
            kind="service-splitter",
            name=service_resolver.name,
            config_json=json.dumps({
                "Splits": [
                    {
                        "Weight": 90,
                        "ServiceSubset": "v1",
                    },
                    {
                        "Weight": 10,
                        "ServiceSubset": "v2",
                    },
                ],
            }))
        service_router = consul.ConfigEntry("service_router",
            kind="service-router",
            name="web",
            config_json=json.dumps({
                "Routes": [{
                    "Match": {
                        "HTTP": {
                            "PathPrefix": "/admin",
                        },
                    },
                    "Destination": {
                        "Service": "admin",
                    },
                }],
            }))
        ingress_gateway = consul.ConfigEntry("ingress_gateway",
            name="us-east-ingress",
            kind="ingress-gateway",
            config_json=json.dumps({
                "TLS": {
                    "Enabled": True,
                },
                "Listeners": [{
                    "Port": 8000,
                    "Protocol": "http",
                    "Services": [{
                        "Name": "*",
                    }],
                }],
            }))
        terminating_gateway = consul.ConfigEntry("terminating_gateway",
            name="us-west-gateway",
            kind="terminating-gateway",
            config_json=json.dumps({
                "Services": [{
                    "Name": "billing",
                }],
            }))
        ```

        ### `service-intentions` config entry

        ```python
        import pulumi
        import json
        import pulumi_consul as consul

        service_intentions = consul.ConfigEntry("service_intentions",
            name="api-service",
            kind="service-intentions",
            config_json=json.dumps({
                "Sources": [
                    {
                        "Action": "allow",
                        "Name": "frontend-webapp",
                        "Precedence": 9,
                        "Type": "consul",
                    },
                    {
                        "Action": "allow",
                        "Name": "nightly-cronjob",
                        "Precedence": 9,
                        "Type": "consul",
                    },
                ],
            }))
        ```

        ```python
        import pulumi
        import json
        import pulumi_consul as consul

        sd = consul.ConfigEntry("sd",
            name="fort-knox",
            kind="service-defaults",
            config_json=json.dumps({
                "Protocol": "http",
            }))
        jwt_provider = consul.ConfigEntry("jwt_provider",
            name="test-provider",
            kind="jwt-provider",
            config_json=json.dumps({
                "Issuer": "test-issuer",
                "JSONWebKeySet": {
                    "Remote": {
                        "URI": "https://127.0.0.1:9091",
                        "FetchAsynchronously": True,
                    },
                },
                "Forwarding": {
                    "HeaderName": "test-token",
                },
            }))
        service_intentions = consul.ConfigEntry("service_intentions",
            name=sd.name,
            kind="service-intentions",
            config_json=pulumi.Output.json_dumps({
                "Sources": [
                    {
                        "Name": "contractor-webapp",
                        "Permissions": [{
                            "Action": "allow",
                            "HTTP": {
                                "Methods": [
                                    "GET",
                                    "HEAD",
                                ],
                                "PathExact": "/healtz",
                            },
                            "JWT": {
                                "Providers": [{
                                    "Name": jwt_provider.name,
                                }],
                            },
                        }],
                        "Precedence": 9,
                        "Type": "consul",
                    },
                    {
                        "Name": "admin-dashboard-webapp",
                        "Permissions": [
                            {
                                "Action": "deny",
                                "HTTP": {
                                    "PathPrefix": "/debugz",
                                },
                            },
                            {
                                "Action": "allow",
                                "HTTP": {
                                    "PathPrefix": "/",
                                },
                            },
                        ],
                        "Precedence": 9,
                        "Type": "consul",
                    },
                ],
            }))
        ```

        ### `exported-services` config entry

        ```python
        import pulumi
        import json
        import pulumi_consul as consul

        exported_services = consul.ConfigEntry("exported_services",
            name="test",
            kind="exported-services",
            config_json=json.dumps({
                "Services": [{
                    "Name": "test",
                    "Namespace": "default",
                    "Consumers": [{
                        "Partition": "default",
                    }],
                }],
            }))
        ```

        ### `mesh` config entry

        ```python
        import pulumi
        import json
        import pulumi_consul as consul

        mesh = consul.ConfigEntry("mesh",
            name="mesh",
            kind="mesh",
            partition="default",
            config_json=json.dumps({
                "TransparentProxy": {
                    "MeshDestinationsOnly": True,
                },
            }))
        ```

        ### `jwt-provider` config entry

        ```python
        import pulumi
        import json
        import pulumi_consul as consul

        jwt_provider = consul.ConfigEntry("jwt_provider",
            name="provider-name",
            kind="jwt-provider",
            config_json=json.dumps({
                "Issuer": "https://your.issuer.com",
                "JSONWebKeySet": {
                    "Remote": {
                        "URI": "https://your-remote.jwks.com",
                        "FetchAsynchronously": True,
                        "CacheDuration": "10s",
                    },
                },
                "Forwarding": {
                    "HeaderName": "test-token",
                },
            }))
        ```

        ## Import

        `consul_config_entry` can be imported using the syntax `<kind>/<name>` if the
        config entry is in the default partition and default namespace, or
        `<partition>/<namespace>/<kind>/<name>` for config entries in a non-default
        partition or namespace:

        ```sh
        $ pulumi import consul:index/configEntry:ConfigEntry service_splitter 816a195f-6cb1-2e8d-92af-3011ae706318
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] config_json: An arbitrary map of configuration values.
        :param pulumi.Input[str] kind: The kind of configuration entry to register.
        :param pulumi.Input[str] name: The name of the configuration entry being registered.
        :param pulumi.Input[str] namespace: The namespace to create the config entry within.
        :param pulumi.Input[str] partition: The partition the config entry is associated with.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ConfigEntryArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        ```python
        import pulumi
        import json
        import pulumi_consul as consul

        proxy_defaults = consul.ConfigEntry("proxy_defaults",
            kind="proxy-defaults",
            name="global",
            config_json=json.dumps({
                "Config": {
                    "local_connect_timeout_ms": 1000,
                    "handshake_timeout_ms": 10000,
                },
            }))
        web = consul.ConfigEntry("web",
            name="web",
            kind="service-defaults",
            config_json=json.dumps({
                "Protocol": "http",
            }))
        admin = consul.ConfigEntry("admin",
            name="admin",
            kind="service-defaults",
            config_json=json.dumps({
                "Protocol": "http",
            }))
        service_resolver = consul.ConfigEntry("service_resolver",
            kind="service-resolver",
            name=web.name,
            config_json=json.dumps({
                "DefaultSubset": "v1",
                "Subsets": {
                    "v1": {
                        "Filter": "Service.Meta.version == v1",
                    },
                    "v2": {
                        "Filter": "Service.Meta.version == v2",
                    },
                },
            }))
        service_splitter = consul.ConfigEntry("service_splitter",
            kind="service-splitter",
            name=service_resolver.name,
            config_json=json.dumps({
                "Splits": [
                    {
                        "Weight": 90,
                        "ServiceSubset": "v1",
                    },
                    {
                        "Weight": 10,
                        "ServiceSubset": "v2",
                    },
                ],
            }))
        service_router = consul.ConfigEntry("service_router",
            kind="service-router",
            name="web",
            config_json=json.dumps({
                "Routes": [{
                    "Match": {
                        "HTTP": {
                            "PathPrefix": "/admin",
                        },
                    },
                    "Destination": {
                        "Service": "admin",
                    },
                }],
            }))
        ingress_gateway = consul.ConfigEntry("ingress_gateway",
            name="us-east-ingress",
            kind="ingress-gateway",
            config_json=json.dumps({
                "TLS": {
                    "Enabled": True,
                },
                "Listeners": [{
                    "Port": 8000,
                    "Protocol": "http",
                    "Services": [{
                        "Name": "*",
                    }],
                }],
            }))
        terminating_gateway = consul.ConfigEntry("terminating_gateway",
            name="us-west-gateway",
            kind="terminating-gateway",
            config_json=json.dumps({
                "Services": [{
                    "Name": "billing",
                }],
            }))
        ```

        ### `service-intentions` config entry

        ```python
        import pulumi
        import json
        import pulumi_consul as consul

        service_intentions = consul.ConfigEntry("service_intentions",
            name="api-service",
            kind="service-intentions",
            config_json=json.dumps({
                "Sources": [
                    {
                        "Action": "allow",
                        "Name": "frontend-webapp",
                        "Precedence": 9,
                        "Type": "consul",
                    },
                    {
                        "Action": "allow",
                        "Name": "nightly-cronjob",
                        "Precedence": 9,
                        "Type": "consul",
                    },
                ],
            }))
        ```

        ```python
        import pulumi
        import json
        import pulumi_consul as consul

        sd = consul.ConfigEntry("sd",
            name="fort-knox",
            kind="service-defaults",
            config_json=json.dumps({
                "Protocol": "http",
            }))
        jwt_provider = consul.ConfigEntry("jwt_provider",
            name="test-provider",
            kind="jwt-provider",
            config_json=json.dumps({
                "Issuer": "test-issuer",
                "JSONWebKeySet": {
                    "Remote": {
                        "URI": "https://127.0.0.1:9091",
                        "FetchAsynchronously": True,
                    },
                },
                "Forwarding": {
                    "HeaderName": "test-token",
                },
            }))
        service_intentions = consul.ConfigEntry("service_intentions",
            name=sd.name,
            kind="service-intentions",
            config_json=pulumi.Output.json_dumps({
                "Sources": [
                    {
                        "Name": "contractor-webapp",
                        "Permissions": [{
                            "Action": "allow",
                            "HTTP": {
                                "Methods": [
                                    "GET",
                                    "HEAD",
                                ],
                                "PathExact": "/healtz",
                            },
                            "JWT": {
                                "Providers": [{
                                    "Name": jwt_provider.name,
                                }],
                            },
                        }],
                        "Precedence": 9,
                        "Type": "consul",
                    },
                    {
                        "Name": "admin-dashboard-webapp",
                        "Permissions": [
                            {
                                "Action": "deny",
                                "HTTP": {
                                    "PathPrefix": "/debugz",
                                },
                            },
                            {
                                "Action": "allow",
                                "HTTP": {
                                    "PathPrefix": "/",
                                },
                            },
                        ],
                        "Precedence": 9,
                        "Type": "consul",
                    },
                ],
            }))
        ```

        ### `exported-services` config entry

        ```python
        import pulumi
        import json
        import pulumi_consul as consul

        exported_services = consul.ConfigEntry("exported_services",
            name="test",
            kind="exported-services",
            config_json=json.dumps({
                "Services": [{
                    "Name": "test",
                    "Namespace": "default",
                    "Consumers": [{
                        "Partition": "default",
                    }],
                }],
            }))
        ```

        ### `mesh` config entry

        ```python
        import pulumi
        import json
        import pulumi_consul as consul

        mesh = consul.ConfigEntry("mesh",
            name="mesh",
            kind="mesh",
            partition="default",
            config_json=json.dumps({
                "TransparentProxy": {
                    "MeshDestinationsOnly": True,
                },
            }))
        ```

        ### `jwt-provider` config entry

        ```python
        import pulumi
        import json
        import pulumi_consul as consul

        jwt_provider = consul.ConfigEntry("jwt_provider",
            name="provider-name",
            kind="jwt-provider",
            config_json=json.dumps({
                "Issuer": "https://your.issuer.com",
                "JSONWebKeySet": {
                    "Remote": {
                        "URI": "https://your-remote.jwks.com",
                        "FetchAsynchronously": True,
                        "CacheDuration": "10s",
                    },
                },
                "Forwarding": {
                    "HeaderName": "test-token",
                },
            }))
        ```

        ## Import

        `consul_config_entry` can be imported using the syntax `<kind>/<name>` if the
        config entry is in the default partition and default namespace, or
        `<partition>/<namespace>/<kind>/<name>` for config entries in a non-default
        partition or namespace:

        ```sh
        $ pulumi import consul:index/configEntry:ConfigEntry service_splitter 816a195f-6cb1-2e8d-92af-3011ae706318
        ```

        :param str resource_name: The name of the resource.
        :param ConfigEntryArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ConfigEntryArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 config_json: Optional[pulumi.Input[str]] = None,
                 kind: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 partition: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ConfigEntryArgs.__new__(ConfigEntryArgs)

            __props__.__dict__["config_json"] = config_json
            if kind is None and not opts.urn:
                raise TypeError("Missing required property 'kind'")
            __props__.__dict__["kind"] = kind
            __props__.__dict__["name"] = name
            __props__.__dict__["namespace"] = namespace
            __props__.__dict__["partition"] = partition
        super(ConfigEntry, __self__).__init__(
            'consul:index/configEntry:ConfigEntry',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            config_json: Optional[pulumi.Input[str]] = None,
            kind: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            namespace: Optional[pulumi.Input[str]] = None,
            partition: Optional[pulumi.Input[str]] = None) -> 'ConfigEntry':
        """
        Get an existing ConfigEntry resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] config_json: An arbitrary map of configuration values.
        :param pulumi.Input[str] kind: The kind of configuration entry to register.
        :param pulumi.Input[str] name: The name of the configuration entry being registered.
        :param pulumi.Input[str] namespace: The namespace to create the config entry within.
        :param pulumi.Input[str] partition: The partition the config entry is associated with.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ConfigEntryState.__new__(_ConfigEntryState)

        __props__.__dict__["config_json"] = config_json
        __props__.__dict__["kind"] = kind
        __props__.__dict__["name"] = name
        __props__.__dict__["namespace"] = namespace
        __props__.__dict__["partition"] = partition
        return ConfigEntry(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="configJson")
    def config_json(self) -> pulumi.Output[Optional[str]]:
        """
        An arbitrary map of configuration values.
        """
        return pulumi.get(self, "config_json")

    @property
    @pulumi.getter
    def kind(self) -> pulumi.Output[str]:
        """
        The kind of configuration entry to register.
        """
        return pulumi.get(self, "kind")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        The name of the configuration entry being registered.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def namespace(self) -> pulumi.Output[Optional[str]]:
        """
        The namespace to create the config entry within.
        """
        return pulumi.get(self, "namespace")

    @property
    @pulumi.getter
    def partition(self) -> pulumi.Output[Optional[str]]:
        """
        The partition the config entry is associated with.
        """
        return pulumi.get(self, "partition")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['ConfigEntryV2ExportedServicesArgs', 'ConfigEntryV2ExportedServices']

@pulumi.input_type
class ConfigEntryV2ExportedServicesArgs:
    def __init__(__self__, *,
                 kind: pulumi.Input[str],
                 partition: pulumi.Input[str],
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 partition_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 peer_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 sameness_group_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 services: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None):
        """
        The set of arguments for constructing a ConfigEntryV2ExportedServices resource.
        :param pulumi.Input[str] kind: The kind of exported services config (ExportedServices, NamespaceExportedServices, PartitionExportedServices).
        :param pulumi.Input[str] partition: The partition the config entry is associated with.
        :param pulumi.Input[str] name: The name of the config entry to read.
        :param pulumi.Input[str] namespace: The namespace the config entry is associated with.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] partition_consumers: The exported service partition consumers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] peer_consumers: The exported service peer consumers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] sameness_group_consumers: The exported service sameness group consumers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] services: The exported services.
        """
        pulumi.set(__self__, "kind", kind)
        pulumi.set(__self__, "partition", partition)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if partition_consumers is not None:
            pulumi.set(__self__, "partition_consumers", partition_consumers)
        if peer_consumers is not None:
            pulumi.set(__self__, "peer_consumers", peer_consumers)
        if sameness_group_consumers is not None:
            pulumi.set(__self__, "sameness_group_consumers", sameness_group_consumers)
        if services is not None:
            pulumi.set(__self__, "services", services)

    @property
    @pulumi.getter
    def kind(self) -> pulumi.Input[str]:
        """
        The kind of exported services config (ExportedServices, NamespaceExportedServices, PartitionExportedServices).
        """
        return pulumi.get(self, "kind")

    @kind.setter
    def kind(self, value: pulumi.Input[str]):
        pulumi.set(self, "kind", value)

    @property
    @pulumi.getter
    def partition(self) -> pulumi.Input[str]:
        """
        The partition the config entry is associated with.
        """
        return pulumi.get(self, "partition")

    @partition.setter
    def partition(self, value: pulumi.Input[str]):
        pulumi.set(self, "partition", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the config entry to read.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        The namespace the config entry is associated with.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter(name="partitionConsumers")
    def partition_consumers(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The exported service partition consumers.
        """
        return pulumi.get(self, "partition_consumers")

    @partition_consumers.setter
    def partition_consumers(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "partition_consumers", value)

    @property
    @pulumi.getter(name="peerConsumers")
    def peer_consumers(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The exported service peer consumers.
        """
        return pulumi.get(self, "peer_consumers")

    @peer_consumers.setter
    def peer_consumers(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "peer_consumers", value)

    @property
    @pulumi.getter(name="samenessGroupConsumers")
    def sameness_group_consumers(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The exported service sameness group consumers.
        """
        return pulumi.get(self, "sameness_group_consumers")

    @sameness_group_consumers.setter
    def sameness_group_consumers(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "sameness_group_consumers", value)

    @property
    @pulumi.getter
    def services(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The exported services.
        """
        return pulumi.get(self, "services")

    @services.setter
    def services(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "services", value)


@pulumi.input_type
class _ConfigEntryV2ExportedServicesState:
    def __init__(__self__, *,
                 kind: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 partition: Optional[pulumi.Input[str]] = None,
                 partition_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 peer_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 sameness_group_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 services: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None):
        """
        Input properties used for looking up and filtering ConfigEntryV2ExportedServices resources.
        :param pulumi.Input[str] kind: The kind of exported services config (ExportedServices, NamespaceExportedServices, PartitionExportedServices).
        :param pulumi.Input[str] name: The name of the config entry to read.
        :param pulumi.Input[str] namespace: The namespace the config entry is associated with.
        :param pulumi.Input[str] partition: The partition the config entry is associated with.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] partition_consumers: The exported service partition consumers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] peer_consumers: The exported service peer consumers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] sameness_group_consumers: The exported service sameness group consumers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] services: The exported services.
        """
        if kind is not None:
            pulumi.set(__self__, "kind", kind)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if partition is not None:
            pulumi.set(__self__, "partition", partition)
        if partition_consumers is not None:
            pulumi.set(__self__, "partition_consumers", partition_consumers)
        if peer_consumers is not None:
            pulumi.set(__self__, "peer_consumers", peer_consumers)
        if sameness_group_consumers is not None:
            pulumi.set(__self__, "sameness_group_consumers", sameness_group_consumers)
        if services is not None:
            pulumi.set(__self__, "services", services)

    @property
    @pulumi.getter
    def kind(self) -> Optional[pulumi.Input[str]]:
        """
        The kind of exported services config (ExportedServices, NamespaceExportedServices, PartitionExportedServices).
        """
        return pulumi.get(self, "kind")

    @kind.setter
    def kind(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "kind", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the config entry to read.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        The namespace the config entry is associated with.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter
    def partition(self) -> Optional[pulumi.Input[str]]:
        """
        The partition the config entry is associated with.
        """
        return pulumi.get(self, "partition")

    @partition.setter
    def partition(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "partition", value)

    @property
    @pulumi.getter(name="partitionConsumers")
    def partition_consumers(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The exported service partition consumers.
        """
        return pulumi.get(self, "partition_consumers")

    @partition_consumers.setter
    def partition_consumers(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "partition_consumers", value)

    @property
    @pulumi.getter(name="peerConsumers")
    def peer_consumers(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The exported service peer consumers.
        """
        return pulumi.get(self, "peer_consumers")

    @peer_consumers.setter
    def peer_consumers(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "peer_consumers", value)

    @property
    @pulumi.getter(name="samenessGroupConsumers")
    def sameness_group_consumers(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The exported service sameness group consumers.
        """
        return pulumi.get(self, "sameness_group_consumers")

    @sameness_group_consumers.setter
    def sameness_group_consumers(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "sameness_group_consumers", value)

    @property
    @pulumi.getter
    def services(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The exported services.
        """
        return pulumi.get(self, "services")

    @services.setter
    def services(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "services", value)


class ConfigEntryV2ExportedServices(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 kind: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 partition: Optional[pulumi.Input[str]] = None,
                 partition_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 peer_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 sameness_group_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 services: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 __props__=None):
        """
        Create a ConfigEntryV2ExportedServices resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] kind: The kind of exported services config (ExportedServices, NamespaceExportedServices, PartitionExportedServices).
        :param pulumi.Input[str] name: The name of the config entry to read.
        :param pulumi.Input[str] namespace: The namespace the config entry is associated with.
        :param pulumi.Input[str] partition: The partition the config entry is associated with.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] partition_consumers: The exported service partition consumers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] peer_consumers: The exported service peer consumers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] sameness_group_consumers: The exported service sameness group consumers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] services: The exported services.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ConfigEntryV2ExportedServicesArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Create a ConfigEntryV2ExportedServices resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param ConfigEntryV2ExportedServicesArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ConfigEntryV2ExportedServicesArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 kind: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 partition: Optional[pulumi.Input[str]] = None,
                 partition_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 peer_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 sameness_group_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 services: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ConfigEntryV2ExportedServicesArgs.__new__(ConfigEntryV2ExportedServicesArgs)

            if kind is None and not opts.urn:
                raise TypeError("Missing required property 'kind'")
            __props__.__dict__["kind"] = kind
            __props__.__dict__["name"] = name
            __props__.__dict__["namespace"] = namespace
            if partition is None and not opts.urn:
                raise TypeError("Missing required property 'partition'")
            __props__.__dict__["partition"] = partition
            __props__.__dict__["partition_consumers"] = partition_consumers
            __props__.__dict__["peer_consumers"] = peer_consumers
            __props__.__dict__["sameness_group_consumers"] = sameness_group_consumers
            __props__.__dict__["services"] = services
        super(ConfigEntryV2ExportedServices, __self__).__init__(
            'consul:index/configEntryV2ExportedServices:ConfigEntryV2ExportedServices',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            kind: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            namespace: Optional[pulumi.Input[str]] = None,
            partition: Optional[pulumi.Input[str]] = None,
            partition_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            peer_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            sameness_group_consumers: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            services: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None) -> 'ConfigEntryV2ExportedServices':
        """
        Get an existing ConfigEntryV2ExportedServices resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] kind: The kind of exported services config (ExportedServices, NamespaceExportedServices, PartitionExportedServices).
        :param pulumi.Input[str] name: The name of the config entry to read.
        :param pulumi.Input[str] namespace: The namespace the config entry is associated with.
        :param pulumi.Input[str] partition: The partition the config entry is associated with.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] partition_consumers: The exported service partition consumers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] peer_consumers: The exported service peer consumers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] sameness_group_consumers: The exported service sameness group consumers.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] services: The exported services.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ConfigEntryV2ExportedServicesState.__new__(_ConfigEntryV2ExportedServicesState)

        __props__.__dict__["kind"] = kind
        __props__.__dict__["name"] = name
        __props__.__dict__["namespace"] = namespace
        __props__.__dict__["partition"] = partition
        __props__.__dict__["partition_consumers"] = partition_consumers
        __props__.__dict__["peer_consumers"] = peer_consumers
        __props__.__dict__["sameness_group_consumers"] = sameness_group_consumers
        __props__.__dict__["services"] = services
        return ConfigEntryV2ExportedServices(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def kind(self) -> pulumi.Output[str]:
        """
        The kind of exported services config (ExportedServices, NamespaceExportedServices, PartitionExportedServices).
        """
        return pulumi.get(self, "kind")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        The name of the config entry to read.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def namespace(self) -> pulumi.Output[Optional[str]]:
        """
        The namespace the config entry is associated with.
        """
        return pulumi.get(self, "namespace")

    @property
    @pulumi.getter
    def partition(self) -> pulumi.Output[str]:
        """
        The partition the config entry is associated with.
        """
        return pulumi.get(self, "partition")

    @property
    @pulumi.getter(name="partitionConsumers")
    def partition_consumers(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        The exported service partition consumers.
        """
        return pulumi.get(self, "partition_consumers")

    @property
    @pulumi.getter(name="peerConsumers")
    def peer_consumers(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        The exported service peer consumers.
        """
        return pulumi.get(self, "peer_consumers")

    @property
    @pulumi.getter(name="samenessGroupConsumers")
    def sameness_group_consumers(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        The exported service sameness group consumers.
        """
        return pulumi.get(self, "sameness_group_consumers")

    @property
    @pulumi.getter
    def services(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        The exported services.
        """
        return pulumi.get(self, "services")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetServiceHealthResult',
    'AwaitableGetServiceHealthResult',
    'get_service_health',
    'get_service_health_output',
]

@pulumi.output_type
class GetServiceHealthResult:
    """
    A collection of values returned by getServiceHealth.
    """
    def __init__(__self__, datacenter=None, filter=None, id=None, name=None, near=None, node_meta=None, passing=None, results=None, tag=None, wait_for=None):
        if datacenter and not isinstance(datacenter, str):
            raise TypeError("Expected argument 'datacenter' to be a str")
        pulumi.set(__self__, "datacenter", datacenter)
        if filter and not isinstance(filter, str):
            raise TypeError("Expected argument 'filter' to be a str")
        pulumi.set(__self__, "filter", filter)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if near and not isinstance(near, str):
            raise TypeError("Expected argument 'near' to be a str")
        pulumi.set(__self__, "near", near)
        if node_meta and not isinstance(node_meta, dict):
            raise TypeError("Expected argument 'node_meta' to be a dict")
        pulumi.set(__self__, "node_meta", node_meta)
        if passing and not isinstance(passing, bool):
            raise TypeError("Expected argument 'passing' to be a bool")
        pulumi.set(__self__, "passing", passing)
        if results and not isinstance(results, list):
            raise TypeError("Expected argument 'results' to be a list")
        pulumi.set(__self__, "results", results)
        if tag and not isinstance(tag, str):
            raise TypeError("Expected argument 'tag' to be a str")
        pulumi.set(__self__, "tag", tag)
        if wait_for and not isinstance(wait_for, str):
            raise TypeError("Expected argument 'wait_for' to be a str")
        pulumi.set(__self__, "wait_for", wait_for)

    @property
    @pulumi.getter
    def datacenter(self) -> Optional[str]:
        """
        The datacenter in which the node is running.
        * [`tagged_addresses`](https://www.consul.io/docs/agent/http/catalog.html#TaggedAddresses) -
        List of explicit LAN and WAN IP addresses for the agent.
        """
        return pulumi.get(self, "datacenter")

    @property
    @pulumi.getter
    def filter(self) -> Optional[str]:
        return pulumi.get(self, "filter")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        The name of this health-check.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def near(self) -> Optional[str]:
        """
        The node to which the result must be sorted to.
        """
        return pulumi.get(self, "near")

    @property
    @pulumi.getter(name="nodeMeta")
    def node_meta(self) -> Optional[Mapping[str, str]]:
        """
        The list of metadata to filter the nodes.
        """
        return pulumi.get(self, "node_meta")

    @property
    @pulumi.getter
    def passing(self) -> Optional[bool]:
        """
        Whether to return only nodes with all checks in the
        passing state.
        """
        return pulumi.get(self, "passing")

    @property
    @pulumi.getter
    def results(self) -> Sequence['outputs.GetServiceHealthResultResult']:
        """
        A list of entries and details about each endpoint advertising a
        service.  Each element in the list has three attributes: `node`, `service` and
        `checks`.  The list of the attributes of each one is detailed below.
        """
        return pulumi.get(self, "results")

    @property
    @pulumi.getter
    def tag(self) -> Optional[str]:
        """
        The name of the tag used to filter the list.
        """
        return pulumi.get(self, "tag")

    @property
    @pulumi.getter(name="waitFor")
    def wait_for(self) -> Optional[str]:
        return pulumi.get(self, "wait_for")


class AwaitableGetServiceHealthResult(GetServiceHealthResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetServiceHealthResult(
            datacenter=self.datacenter,
            filter=self.filter,
            id=self.id,
            name=self.name,
            near=self.near,
            node_meta=self.node_meta,
            passing=self.passing,
            results=self.results,
            tag=self.tag,
            wait_for=self.wait_for)


def get_service_health(datacenter: Optional[str] = None,
                       filter: Optional[str] = None,
                       name: Optional[str] = None,
                       near: Optional[str] = None,
                       node_meta: Optional[Mapping[str, str]] = None,
                       passing: Optional[bool] = None,
                       tag: Optional[str] = None,
                       wait_for: Optional[str] = None,
                       opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetServiceHealthResult:
    """
    `get_service_health` can be used to get the list of the instances that
    are currently healthy, according to their associated  health-checks.
    The result includes the list of service instances, the node associated to each
    instance and its health-checks.

    This resource is likely to change as frequently as the health-checks are being
    updated, you should expect different results in a frequent basis.


    :param str datacenter: The Consul datacenter to query.
    :param str filter: A filter expression to refine the list of results, see
           https://www.consul.io/api-docs/features/filtering and https://www.consul.io/api-docs/health#filtering-2.
    :param str name: The service name to select.
    :param str near: Specifies a node name to sort the node list in ascending order
           based on the estimated round trip time from that node.
    :param Mapping[str, str] node_meta: Filter the results to nodes with the specified key/value
           pairs.
    :param bool passing: Whether to return only nodes with all checks in the
           passing state. Defaults to `true`.
    :param str tag: A single tag that can be used to filter the list to return
           based on a single matching tag.
    """
    __args__ = dict()
    __args__['datacenter'] = datacenter
    __args__['filter'] = filter
    __args__['name'] = name
    __args__['near'] = near
    __args__['nodeMeta'] = node_meta
    __args__['passing'] = passing
    __args__['tag'] = tag
    __args__['waitFor'] = wait_for
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('consul:index/getServiceHealth:getServiceHealth', __args__, opts=opts, typ=GetServiceHealthResult).value

    return AwaitableGetServiceHealthResult(
        datacenter=pulumi.get(__ret__, 'datacenter'),
        filter=pulumi.get(__ret__, 'filter'),
        id=pulumi.get(__ret__, 'id'),
        name=pulumi.get(__ret__, 'name'),
        near=pulumi.get(__ret__, 'near'),
        node_meta=pulumi.get(__ret__, 'node_meta'),
        passing=pulumi.get(__ret__, 'passing'),
        results=pulumi.get(__ret__, 'results'),
        tag=pulumi.get(__ret__, 'tag'),
        wait_for=pulumi.get(__ret__, 'wait_for'))
def get_service_health_output(datacenter: Optional[pulumi.Input[Optional[str]]] = None,
                              filter: Optional[pulumi.Input[Optional[str]]] = None,
                              name: Optional[pulumi.Input[str]] = None,
                              near: Optional[pulumi.Input[Optional[str]]] = None,
                              node_meta: Optional[pulumi.Input[Optional[Mapping[str, str]]]] = None,
                              passing: Optional[pulumi.Input[Optional[bool]]] = None,
                              tag: Optional[pulumi.Input[Optional[str]]] = None,
                              wait_for: Optional[pulumi.Input[Optional[str]]] = None,
                              opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetServiceHealthResult]:
    """
    `get_service_health` can be used to get the list of the instances that
    are currently healthy, according to their associated  health-checks.
    The result includes the list of service instances, the node associated to each
    instance and its health-checks.

    This resource is likely to change as frequently as the health-checks are being
    updated, you should expect different results in a frequent basis.


    :param str datacenter: The Consul datacenter to query.
    :param str filter: A filter expression to refine the list of results, see
           https://www.consul.io/api-docs/features/filtering and https://www.consul.io/api-docs/health#filtering-2.
    :param str name: The service name to select.
    :param str near: Specifies a node name to sort the node list in ascending order
           based on the estimated round trip time from that node.
    :param Mapping[str, str] node_meta: Filter the results to nodes with the specified key/value
           pairs.
    :param bool passing: Whether to return only nodes with all checks in the
           passing state. Defaults to `true`.
    :param str tag: A single tag that can be used to filter the list to return
           based on a single matching tag.
    """
    __args__ = dict()
    __args__['datacenter'] = datacenter
    __args__['filter'] = filter
    __args__['name'] = name
    __args__['near'] = near
    __args__['nodeMeta'] = node_meta
    __args__['passing'] = passing
    __args__['tag'] = tag
    __args__['waitFor'] = wait_for
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('consul:index/getServiceHealth:getServiceHealth', __args__, opts=opts, typ=GetServiceHealthResult)
    return __ret__.apply(lambda __response__: GetServiceHealthResult(
        datacenter=pulumi.get(__response__, 'datacenter'),
        filter=pulumi.get(__response__, 'filter'),
        id=pulumi.get(__response__, 'id'),
        name=pulumi.get(__response__, 'name'),
        near=pulumi.get(__response__, 'near'),
        node_meta=pulumi.get(__response__, 'node_meta'),
        passing=pulumi.get(__response__, 'passing'),
        results=pulumi.get(__response__, 'results'),
        tag=pulumi.get(__response__, 'tag'),
        wait_for=pulumi.get(__response__, 'wait_for')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = [
    'GetAgentConfigResult',
    'AwaitableGetAgentConfigResult',
    'get_agent_config',
    'get_agent_config_output',
]

@pulumi.output_type
class GetAgentConfigResult:
    """
    A collection of values returned by getAgentConfig.
    """
    def __init__(__self__, datacenter=None, id=None, node_id=None, node_name=None, revision=None, server=None, version=None):
        if datacenter and not isinstance(datacenter, str):
            raise TypeError("Expected argument 'datacenter' to be a str")
        pulumi.set(__self__, "datacenter", datacenter)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if node_id and not isinstance(node_id, str):
            raise TypeError("Expected argument 'node_id' to be a str")
        pulumi.set(__self__, "node_id", node_id)
        if node_name and not isinstance(node_name, str):
            raise TypeError("Expected argument 'node_name' to be a str")
        pulumi.set(__self__, "node_name", node_name)
        if revision and not isinstance(revision, str):
            raise TypeError("Expected argument 'revision' to be a str")
        pulumi.set(__self__, "revision", revision)
        if server and not isinstance(server, bool):
            raise TypeError("Expected argument 'server' to be a bool")
        pulumi.set(__self__, "server", server)
        if version and not isinstance(version, str):
            raise TypeError("Expected argument 'version' to be a str")
        pulumi.set(__self__, "version", version)

    @property
    @pulumi.getter
    def datacenter(self) -> str:
        """
        The datacenter the agent is running in
        """
        return pulumi.get(self, "datacenter")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="nodeId")
    def node_id(self) -> str:
        """
        The ID of the node the agent is running on
        """
        return pulumi.get(self, "node_id")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> str:
        """
        The name of the node the agent is running on
        """
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter
    def revision(self) -> str:
        """
        The first 9 characters of the VCS revision of the build of Consul that is running
        """
        return pulumi.get(self, "revision")

    @property
    @pulumi.getter
    def server(self) -> bool:
        """
        Boolean if the agent is a server or not
        """
        return pulumi.get(self, "server")

    @property
    @pulumi.getter
    def version(self) -> str:
        """
        The version of the build of Consul that is running
        """
        return pulumi.get(self, "version")


class AwaitableGetAgentConfigResult(GetAgentConfigResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAgentConfigResult(
            datacenter=self.datacenter,
            id=self.id,
            node_id=self.node_id,
            node_name=self.node_name,
            revision=self.revision,
            server=self.server,
            version=self.version)


def get_agent_config(opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetAgentConfigResult:
    """
    > **Note:** The `get_agent_config` resource differs from [`get_agent_self`](https://www.terraform.io/docs/providers/consul/d/agent_self.html),
    providing less information but utilizing stable APIs. `get_agent_self` will be
    deprecated in a future release.

    The `get_agent_config` data source returns
    [configuration data](https://www.consul.io/api/agent.html#read-configuration)
    from the agent specified in the `provider`.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_consul as consul

    remote_agent = consul.get_agent_config()
    pulumi.export("consulVersion", remote_agent.version)
    ```
    """
    __args__ = dict()
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('consul:index/getAgentConfig:getAgentConfig', __args__, opts=opts, typ=GetAgentConfigResult).value

    return AwaitableGetAgentConfigResult(
        datacenter=pulumi.get(__ret__, 'datacenter'),
        id=pulumi.get(__ret__, 'id'),
        node_id=pulumi.get(__ret__, 'node_id'),
        node_name=pulumi.get(__ret__, 'node_name'),
        revision=pulumi.get(__ret__, 'revision'),
        server=pulumi.get(__ret__, 'server'),
        version=pulumi.get(__ret__, 'version'))
def get_agent_config_output(opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetAgentConfigResult]:
    """
    > **Note:** The `get_agent_config` resource differs from [`get_agent_self`](https://www.terraform.io/docs/providers/consul/d/agent_self.html),
    providing less information but utilizing stable APIs. `get_agent_self` will be
    deprecated in a future release.

    The `get_agent_config` data source returns
    [configuration data](https://www.consul.io/api/agent.html#read-configuration)
    from the agent specified in the `provider`.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_consul as consul

    remote_agent = consul.get_agent_config()
    pulumi.export("consulVersion", remote_agent.version)
    ```
    """
    __args__ = dict()
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('consul:index/getAgentConfig:getAgentConfig', __args__, opts=opts, typ=GetAgentConfigResult)
    return __ret__.apply(lambda __response__: GetAgentConfigResult(
        datacenter=pulumi.get(__response__, 'datacenter'),
        id=pulumi.get(__response__, 'id'),
        node_id=pulumi.get(__response__, 'node_id'),
        node_name=pulumi.get(__response__, 'node_name'),
        revision=pulumi.get(__response__, 'revision'),
        server=pulumi.get(__response__, 'server'),
        version=pulumi.get(__response__, 'version')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = [
    'GetCatalogServicesResult',
    'AwaitableGetCatalogServicesResult',
    'get_catalog_services',
    'get_catalog_services_output',
]

warnings.warn("""getCatalogServices has been deprecated in favor of getServices""", DeprecationWarning)

@pulumi.output_type
class GetCatalogServicesResult:
    """
    A collection of values returned by getCatalogServices.
    """
    def __init__(__self__, datacenter=None, id=None, names=None, query_options=None, services=None, tags=None):
        if datacenter and not isinstance(datacenter, str):
            raise TypeError("Expected argument 'datacenter' to be a str")
        pulumi.set(__self__, "datacenter", datacenter)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if names and not isinstance(names, list):
            raise TypeError("Expected argument 'names' to be a list")
        pulumi.set(__self__, "names", names)
        if query_options and not isinstance(query_options, list):
            raise TypeError("Expected argument 'query_options' to be a list")
        pulumi.set(__self__, "query_options", query_options)
        if services and not isinstance(services, dict):
            raise TypeError("Expected argument 'services' to be a dict")
        pulumi.set(__self__, "services", services)
        if tags and not isinstance(tags, dict):
            raise TypeError("Expected argument 'tags' to be a dict")
        pulumi.set(__self__, "tags", tags)

    @property
    @pulumi.getter
    def datacenter(self) -> str:
        """
        The datacenter the keys are being read from to.
        """
        return pulumi.get(self, "datacenter")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def names(self) -> Sequence[str]:
        return pulumi.get(self, "names")

    @property
    @pulumi.getter(name="queryOptions")
    def query_options(self) -> Optional[Sequence['outputs.GetCatalogServicesQueryOptionResult']]:
        return pulumi.get(self, "query_options")

    @property
    @pulumi.getter
    def services(self) -> Mapping[str, str]:
        return pulumi.get(self, "services")

    @property
    @pulumi.getter
    def tags(self) -> Mapping[str, str]:
        """
        A map of the tags found for each service.  If more than one service
        shares the same tag, unique service names will be joined by whitespace (this
        is the inverse of `services` and can be used to lookup the services that match
        a single tag).
        """
        return pulumi.get(self, "tags")


class AwaitableGetCatalogServicesResult(GetCatalogServicesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetCatalogServicesResult(
            datacenter=self.datacenter,
            id=self.id,
            names=self.names,
            query_options=self.query_options,
            services=self.services,
            tags=self.tags)


def get_catalog_services(query_options: Optional[Sequence[Union['GetCatalogServicesQueryOptionArgs', 'GetCatalogServicesQueryOptionArgsDict']]] = None,
                         opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetCatalogServicesResult:
    """
    The `get_services` data source returns a list of Consul services that
    have been registered with the Consul cluster in a given datacenter.  By
    specifying a different datacenter in the `query_options` it is possible to
    retrieve a list of services from a different WAN-attached Consul datacenter.

    This data source is different from the `Service` (singular) data
    source, which provides a detailed response about a specific Consul service.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_consul as consul
    import pulumi_example as example
    import pulumi_std as std

    read_dc1 = consul.get_services(query_options=[{
        "datacenter": "dc1",
    }])
    # Set the description to a whitespace delimited list of the services
    app = example.index.Resource("app", description=std.join(separator= ,
        input=names).result)
    ```


    :param Sequence[Union['GetCatalogServicesQueryOptionArgs', 'GetCatalogServicesQueryOptionArgsDict']] query_options: See below.
    """
    pulumi.log.warn("""get_catalog_services is deprecated: getCatalogServices has been deprecated in favor of getServices""")
    __args__ = dict()
    __args__['queryOptions'] = query_options
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('consul:index/getCatalogServices:getCatalogServices', __args__, opts=opts, typ=GetCatalogServicesResult).value

    return AwaitableGetCatalogServicesResult(
        datacenter=pulumi.get(__ret__, 'datacenter'),
        id=pulumi.get(__ret__, 'id'),
        names=pulumi.get(__ret__, 'names'),
        query_options=pulumi.get(__ret__, 'query_options'),
        services=pulumi.get(__ret__, 'services'),
        tags=pulumi.get(__ret__, 'tags'))
def get_catalog_services_output(query_options: Optional[pulumi.Input[Optional[Sequence[Union['GetCatalogServicesQueryOptionArgs', 'GetCatalogServicesQueryOptionArgsDict']]]]] = None,
                                opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetCatalogServicesResult]:
    """
    The `get_services` data source returns a list of Consul services that
    have been registered with the Consul cluster in a given datacenter.  By
    specifying a different datacenter in the `query_options` it is possible to
    retrieve a list of services from a different WAN-attached Consul datacenter.

    This data source is different from the `Service` (singular) data
    source, which provides a detailed response about a specific Consul service.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_consul as consul
    import pulumi_example as example
    import pulumi_std as std

    read_dc1 = consul.get_services(query_options=[{
        "datacenter": "dc1",
    }])
    # Set the description to a whitespace delimited list of the services
    app = example.index.Resource("app", description=std.join(separator= ,
        input=names).result)
    ```


    :param Sequence[Union['GetCatalogServicesQueryOptionArgs', 'GetCatalogServicesQueryOptionArgsDict']] query_options: See below.
    """
    pulumi.log.warn("""get_catalog_services is deprecated: getCatalogServices has been deprecated in favor of getServices""")
    __args__ = dict()
    __args__['queryOptions'] = query_options
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('consul:index/getCatalogServices:getCatalogServices', __args__, opts=opts, typ=GetCatalogServicesResult)
    return __ret__.apply(lambda __response__: GetCatalogServicesResult(
        datacenter=pulumi.get(__response__, 'datacenter'),
        id=pulumi.get(__response__, 'id'),
        names=pulumi.get(__response__, 'names'),
        query_options=pulumi.get(__response__, 'query_options'),
        services=pulumi.get(__response__, 'services'),
        tags=pulumi.get(__response__, 'tags')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['ConfigEntryServiceDefaultsArgs', 'ConfigEntryServiceDefaults']

@pulumi.input_type
class ConfigEntryServiceDefaultsArgs:
    def __init__(__self__, *,
                 exposes: pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsExposeArgs']]],
                 protocol: pulumi.Input[str],
                 balance_inbound_connections: Optional[pulumi.Input[str]] = None,
                 destinations: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsDestinationArgs']]]] = None,
                 envoy_extensions: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsEnvoyExtensionArgs']]]] = None,
                 external_sni: Optional[pulumi.Input[str]] = None,
                 local_connect_timeout_ms: Optional[pulumi.Input[int]] = None,
                 local_request_timeout_ms: Optional[pulumi.Input[int]] = None,
                 max_inbound_connections: Optional[pulumi.Input[int]] = None,
                 mesh_gateways: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsMeshGatewayArgs']]]] = None,
                 meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 mode: Optional[pulumi.Input[str]] = None,
                 mutual_tls_mode: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 partition: Optional[pulumi.Input[str]] = None,
                 transparent_proxies: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsTransparentProxyArgs']]]] = None,
                 upstream_configs: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsUpstreamConfigArgs']]]] = None):
        """
        The set of arguments for constructing a ConfigEntryServiceDefaults resource.
        :param pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsExposeArgs']]] exposes: Specifies default configurations for exposing HTTP paths through Envoy.
        :param pulumi.Input[str] protocol: Specifies the default protocol for the service.
        :param pulumi.Input[str] balance_inbound_connections: Specifies the strategy for allocating inbound connections to the service across Envoy proxy threads.
        :param pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsDestinationArgs']]] destinations: Configures the destination for service traffic through terminating gateways.
        :param pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsEnvoyExtensionArgs']]] envoy_extensions: List of extensions to modify Envoy proxy configuration.
        :param pulumi.Input[str] external_sni: Specifies the TLS server name indication (SNI) when federating with an external system.
        :param pulumi.Input[int] local_connect_timeout_ms: Specifies the number of milliseconds allowed for establishing connections to the local application instance before timing out.
        :param pulumi.Input[int] local_request_timeout_ms: Specifies the timeout for HTTP requests to the local application instance.
        :param pulumi.Input[int] max_inbound_connections: Specifies the maximum number of concurrent inbound connections to each service instance.
        :param pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsMeshGatewayArgs']]] mesh_gateways: Specifies the default mesh gateway mode field for the service.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] meta: Specifies a set of custom key-value pairs to add to the Consul KV store.
        :param pulumi.Input[str] mode: Specifies a mode for how the service directs inbound and outbound traffic.
        :param pulumi.Input[str] mutual_tls_mode: Controls whether mutual TLS is required for incoming connections to this service. This setting is only supported for services with transparent proxy enabled.
        :param pulumi.Input[str] name: Specifies the name of the service you are setting the defaults for.
        :param pulumi.Input[str] namespace: Specifies the Consul namespace that the configuration entry applies to.
        :param pulumi.Input[str] partition: Specifies the name of the name of the Consul admin partition that the configuration entry applies to. Refer to Admin Partitions for additional information.
        :param pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsTransparentProxyArgs']]] transparent_proxies: Controls configurations specific to proxies in transparent mode. Refer to Transparent Proxy Mode for additional information.
        :param pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsUpstreamConfigArgs']]] upstream_configs: Controls default upstream connection settings and custom overrides for individual upstream services.
        """
        pulumi.set(__self__, "exposes", exposes)
        pulumi.set(__self__, "protocol", protocol)
        if balance_inbound_connections is not None:
            pulumi.set(__self__, "balance_inbound_connections", balance_inbound_connections)
        if destinations is not None:
            pulumi.set(__self__, "destinations", destinations)
        if envoy_extensions is not None:
            pulumi.set(__self__, "envoy_extensions", envoy_extensions)
        if external_sni is not None:
            pulumi.set(__self__, "external_sni", external_sni)
        if local_connect_timeout_ms is not None:
            pulumi.set(__self__, "local_connect_timeout_ms", local_connect_timeout_ms)
        if local_request_timeout_ms is not None:
            pulumi.set(__self__, "local_request_timeout_ms", local_request_timeout_ms)
        if max_inbound_connections is not None:
            pulumi.set(__self__, "max_inbound_connections", max_inbound_connections)
        if mesh_gateways is not None:
            pulumi.set(__self__, "mesh_gateways", mesh_gateways)
        if meta is not None:
            pulumi.set(__self__, "meta", meta)
        if mode is not None:
            pulumi.set(__self__, "mode", mode)
        if mutual_tls_mode is not None:
            pulumi.set(__self__, "mutual_tls_mode", mutual_tls_mode)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if partition is not None:
            pulumi.set(__self__, "partition", partition)
        if transparent_proxies is not None:
            pulumi.set(__self__, "transparent_proxies", transparent_proxies)
        if upstream_configs is not None:
            pulumi.set(__self__, "upstream_configs", upstream_configs)

    @property
    @pulumi.getter
    def exposes(self) -> pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsExposeArgs']]]:
        """
        Specifies default configurations for exposing HTTP paths through Envoy.
        """
        return pulumi.get(self, "exposes")

    @exposes.setter
    def exposes(self, value: pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsExposeArgs']]]):
        pulumi.set(self, "exposes", value)

    @property
    @pulumi.getter
    def protocol(self) -> pulumi.Input[str]:
        """
        Specifies the default protocol for the service.
        """
        return pulumi.get(self, "protocol")

    @protocol.setter
    def protocol(self, value: pulumi.Input[str]):
        pulumi.set(self, "protocol", value)

    @property
    @pulumi.getter(name="balanceInboundConnections")
    def balance_inbound_connections(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the strategy for allocating inbound connections to the service across Envoy proxy threads.
        """
        return pulumi.get(self, "balance_inbound_connections")

    @balance_inbound_connections.setter
    def balance_inbound_connections(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "balance_inbound_connections", value)

    @property
    @pulumi.getter
    def destinations(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsDestinationArgs']]]]:
        """
        Configures the destination for service traffic through terminating gateways.
        """
        return pulumi.get(self, "destinations")

    @destinations.setter
    def destinations(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsDestinationArgs']]]]):
        pulumi.set(self, "destinations", value)

    @property
    @pulumi.getter(name="envoyExtensions")
    def envoy_extensions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsEnvoyExtensionArgs']]]]:
        """
        List of extensions to modify Envoy proxy configuration.
        """
        return pulumi.get(self, "envoy_extensions")

    @envoy_extensions.setter
    def envoy_extensions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsEnvoyExtensionArgs']]]]):
        pulumi.set(self, "envoy_extensions", value)

    @property
    @pulumi.getter(name="externalSni")
    def external_sni(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the TLS server name indication (SNI) when federating with an external system.
        """
        return pulumi.get(self, "external_sni")

    @external_sni.setter
    def external_sni(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "external_sni", value)

    @property
    @pulumi.getter(name="localConnectTimeoutMs")
    def local_connect_timeout_ms(self) -> Optional[pulumi.Input[int]]:
        """
        Specifies the number of milliseconds allowed for establishing connections to the local application instance before timing out.
        """
        return pulumi.get(self, "local_connect_timeout_ms")

    @local_connect_timeout_ms.setter
    def local_connect_timeout_ms(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "local_connect_timeout_ms", value)

    @property
    @pulumi.getter(name="localRequestTimeoutMs")
    def local_request_timeout_ms(self) -> Optional[pulumi.Input[int]]:
        """
        Specifies the timeout for HTTP requests to the local application instance.
        """
        return pulumi.get(self, "local_request_timeout_ms")

    @local_request_timeout_ms.setter
    def local_request_timeout_ms(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "local_request_timeout_ms", value)

    @property
    @pulumi.getter(name="maxInboundConnections")
    def max_inbound_connections(self) -> Optional[pulumi.Input[int]]:
        """
        Specifies the maximum number of concurrent inbound connections to each service instance.
        """
        return pulumi.get(self, "max_inbound_connections")

    @max_inbound_connections.setter
    def max_inbound_connections(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "max_inbound_connections", value)

    @property
    @pulumi.getter(name="meshGateways")
    def mesh_gateways(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsMeshGatewayArgs']]]]:
        """
        Specifies the default mesh gateway mode field for the service.
        """
        return pulumi.get(self, "mesh_gateways")

    @mesh_gateways.setter
    def mesh_gateways(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsMeshGatewayArgs']]]]):
        pulumi.set(self, "mesh_gateways", value)

    @property
    @pulumi.getter
    def meta(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Specifies a set of custom key-value pairs to add to the Consul KV store.
        """
        return pulumi.get(self, "meta")

    @meta.setter
    def meta(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "meta", value)

    @property
    @pulumi.getter
    def mode(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies a mode for how the service directs inbound and outbound traffic.
        """
        return pulumi.get(self, "mode")

    @mode.setter
    def mode(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "mode", value)

    @property
    @pulumi.getter(name="mutualTlsMode")
    def mutual_tls_mode(self) -> Optional[pulumi.Input[str]]:
        """
        Controls whether mutual TLS is required for incoming connections to this service. This setting is only supported for services with transparent proxy enabled.
        """
        return pulumi.get(self, "mutual_tls_mode")

    @mutual_tls_mode.setter
    def mutual_tls_mode(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "mutual_tls_mode", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the name of the service you are setting the defaults for.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the Consul namespace that the configuration entry applies to.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter
    def partition(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the name of the name of the Consul admin partition that the configuration entry applies to. Refer to Admin Partitions for additional information.
        """
        return pulumi.get(self, "partition")

    @partition.setter
    def partition(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "partition", value)

    @property
    @pulumi.getter(name="transparentProxies")
    def transparent_proxies(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsTransparentProxyArgs']]]]:
        """
        Controls configurations specific to proxies in transparent mode. Refer to Transparent Proxy Mode for additional information.
        """
        return pulumi.get(self, "transparent_proxies")

    @transparent_proxies.setter
    def transparent_proxies(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsTransparentProxyArgs']]]]):
        pulumi.set(self, "transparent_proxies", value)

    @property
    @pulumi.getter(name="upstreamConfigs")
    def upstream_configs(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsUpstreamConfigArgs']]]]:
        """
        Controls default upstream connection settings and custom overrides for individual upstream services.
        """
        return pulumi.get(self, "upstream_configs")

    @upstream_configs.setter
    def upstream_configs(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsUpstreamConfigArgs']]]]):
        pulumi.set(self, "upstream_configs", value)


@pulumi.input_type
class _ConfigEntryServiceDefaultsState:
    def __init__(__self__, *,
                 balance_inbound_connections: Optional[pulumi.Input[str]] = None,
                 destinations: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsDestinationArgs']]]] = None,
                 envoy_extensions: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsEnvoyExtensionArgs']]]] = None,
                 exposes: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsExposeArgs']]]] = None,
                 external_sni: Optional[pulumi.Input[str]] = None,
                 local_connect_timeout_ms: Optional[pulumi.Input[int]] = None,
                 local_request_timeout_ms: Optional[pulumi.Input[int]] = None,
                 max_inbound_connections: Optional[pulumi.Input[int]] = None,
                 mesh_gateways: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsMeshGatewayArgs']]]] = None,
                 meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 mode: Optional[pulumi.Input[str]] = None,
                 mutual_tls_mode: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 partition: Optional[pulumi.Input[str]] = None,
                 protocol: Optional[pulumi.Input[str]] = None,
                 transparent_proxies: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsTransparentProxyArgs']]]] = None,
                 upstream_configs: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsUpstreamConfigArgs']]]] = None):
        """
        Input properties used for looking up and filtering ConfigEntryServiceDefaults resources.
        :param pulumi.Input[str] balance_inbound_connections: Specifies the strategy for allocating inbound connections to the service across Envoy proxy threads.
        :param pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsDestinationArgs']]] destinations: Configures the destination for service traffic through terminating gateways.
        :param pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsEnvoyExtensionArgs']]] envoy_extensions: List of extensions to modify Envoy proxy configuration.
        :param pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsExposeArgs']]] exposes: Specifies default configurations for exposing HTTP paths through Envoy.
        :param pulumi.Input[str] external_sni: Specifies the TLS server name indication (SNI) when federating with an external system.
        :param pulumi.Input[int] local_connect_timeout_ms: Specifies the number of milliseconds allowed for establishing connections to the local application instance before timing out.
        :param pulumi.Input[int] local_request_timeout_ms: Specifies the timeout for HTTP requests to the local application instance.
        :param pulumi.Input[int] max_inbound_connections: Specifies the maximum number of concurrent inbound connections to each service instance.
        :param pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsMeshGatewayArgs']]] mesh_gateways: Specifies the default mesh gateway mode field for the service.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] meta: Specifies a set of custom key-value pairs to add to the Consul KV store.
        :param pulumi.Input[str] mode: Specifies a mode for how the service directs inbound and outbound traffic.
        :param pulumi.Input[str] mutual_tls_mode: Controls whether mutual TLS is required for incoming connections to this service. This setting is only supported for services with transparent proxy enabled.
        :param pulumi.Input[str] name: Specifies the name of the service you are setting the defaults for.
        :param pulumi.Input[str] namespace: Specifies the Consul namespace that the configuration entry applies to.
        :param pulumi.Input[str] partition: Specifies the name of the name of the Consul admin partition that the configuration entry applies to. Refer to Admin Partitions for additional information.
        :param pulumi.Input[str] protocol: Specifies the default protocol for the service.
        :param pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsTransparentProxyArgs']]] transparent_proxies: Controls configurations specific to proxies in transparent mode. Refer to Transparent Proxy Mode for additional information.
        :param pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsUpstreamConfigArgs']]] upstream_configs: Controls default upstream connection settings and custom overrides for individual upstream services.
        """
        if balance_inbound_connections is not None:
            pulumi.set(__self__, "balance_inbound_connections", balance_inbound_connections)
        if destinations is not None:
            pulumi.set(__self__, "destinations", destinations)
        if envoy_extensions is not None:
            pulumi.set(__self__, "envoy_extensions", envoy_extensions)
        if exposes is not None:
            pulumi.set(__self__, "exposes", exposes)
        if external_sni is not None:
            pulumi.set(__self__, "external_sni", external_sni)
        if local_connect_timeout_ms is not None:
            pulumi.set(__self__, "local_connect_timeout_ms", local_connect_timeout_ms)
        if local_request_timeout_ms is not None:
            pulumi.set(__self__, "local_request_timeout_ms", local_request_timeout_ms)
        if max_inbound_connections is not None:
            pulumi.set(__self__, "max_inbound_connections", max_inbound_connections)
        if mesh_gateways is not None:
            pulumi.set(__self__, "mesh_gateways", mesh_gateways)
        if meta is not None:
            pulumi.set(__self__, "meta", meta)
        if mode is not None:
            pulumi.set(__self__, "mode", mode)
        if mutual_tls_mode is not None:
            pulumi.set(__self__, "mutual_tls_mode", mutual_tls_mode)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if namespace is not None:
            pulumi.set(__self__, "namespace", namespace)
        if partition is not None:
            pulumi.set(__self__, "partition", partition)
        if protocol is not None:
            pulumi.set(__self__, "protocol", protocol)
        if transparent_proxies is not None:
            pulumi.set(__self__, "transparent_proxies", transparent_proxies)
        if upstream_configs is not None:
            pulumi.set(__self__, "upstream_configs", upstream_configs)

    @property
    @pulumi.getter(name="balanceInboundConnections")
    def balance_inbound_connections(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the strategy for allocating inbound connections to the service across Envoy proxy threads.
        """
        return pulumi.get(self, "balance_inbound_connections")

    @balance_inbound_connections.setter
    def balance_inbound_connections(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "balance_inbound_connections", value)

    @property
    @pulumi.getter
    def destinations(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsDestinationArgs']]]]:
        """
        Configures the destination for service traffic through terminating gateways.
        """
        return pulumi.get(self, "destinations")

    @destinations.setter
    def destinations(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsDestinationArgs']]]]):
        pulumi.set(self, "destinations", value)

    @property
    @pulumi.getter(name="envoyExtensions")
    def envoy_extensions(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsEnvoyExtensionArgs']]]]:
        """
        List of extensions to modify Envoy proxy configuration.
        """
        return pulumi.get(self, "envoy_extensions")

    @envoy_extensions.setter
    def envoy_extensions(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsEnvoyExtensionArgs']]]]):
        pulumi.set(self, "envoy_extensions", value)

    @property
    @pulumi.getter
    def exposes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsExposeArgs']]]]:
        """
        Specifies default configurations for exposing HTTP paths through Envoy.
        """
        return pulumi.get(self, "exposes")

    @exposes.setter
    def exposes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsExposeArgs']]]]):
        pulumi.set(self, "exposes", value)

    @property
    @pulumi.getter(name="externalSni")
    def external_sni(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the TLS server name indication (SNI) when federating with an external system.
        """
        return pulumi.get(self, "external_sni")

    @external_sni.setter
    def external_sni(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "external_sni", value)

    @property
    @pulumi.getter(name="localConnectTimeoutMs")
    def local_connect_timeout_ms(self) -> Optional[pulumi.Input[int]]:
        """
        Specifies the number of milliseconds allowed for establishing connections to the local application instance before timing out.
        """
        return pulumi.get(self, "local_connect_timeout_ms")

    @local_connect_timeout_ms.setter
    def local_connect_timeout_ms(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "local_connect_timeout_ms", value)

    @property
    @pulumi.getter(name="localRequestTimeoutMs")
    def local_request_timeout_ms(self) -> Optional[pulumi.Input[int]]:
        """
        Specifies the timeout for HTTP requests to the local application instance.
        """
        return pulumi.get(self, "local_request_timeout_ms")

    @local_request_timeout_ms.setter
    def local_request_timeout_ms(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "local_request_timeout_ms", value)

    @property
    @pulumi.getter(name="maxInboundConnections")
    def max_inbound_connections(self) -> Optional[pulumi.Input[int]]:
        """
        Specifies the maximum number of concurrent inbound connections to each service instance.
        """
        return pulumi.get(self, "max_inbound_connections")

    @max_inbound_connections.setter
    def max_inbound_connections(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "max_inbound_connections", value)

    @property
    @pulumi.getter(name="meshGateways")
    def mesh_gateways(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsMeshGatewayArgs']]]]:
        """
        Specifies the default mesh gateway mode field for the service.
        """
        return pulumi.get(self, "mesh_gateways")

    @mesh_gateways.setter
    def mesh_gateways(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsMeshGatewayArgs']]]]):
        pulumi.set(self, "mesh_gateways", value)

    @property
    @pulumi.getter
    def meta(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Specifies a set of custom key-value pairs to add to the Consul KV store.
        """
        return pulumi.get(self, "meta")

    @meta.setter
    def meta(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "meta", value)

    @property
    @pulumi.getter
    def mode(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies a mode for how the service directs inbound and outbound traffic.
        """
        return pulumi.get(self, "mode")

    @mode.setter
    def mode(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "mode", value)

    @property
    @pulumi.getter(name="mutualTlsMode")
    def mutual_tls_mode(self) -> Optional[pulumi.Input[str]]:
        """
        Controls whether mutual TLS is required for incoming connections to this service. This setting is only supported for services with transparent proxy enabled.
        """
        return pulumi.get(self, "mutual_tls_mode")

    @mutual_tls_mode.setter
    def mutual_tls_mode(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "mutual_tls_mode", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the name of the service you are setting the defaults for.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def namespace(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the Consul namespace that the configuration entry applies to.
        """
        return pulumi.get(self, "namespace")

    @namespace.setter
    def namespace(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "namespace", value)

    @property
    @pulumi.getter
    def partition(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the name of the name of the Consul admin partition that the configuration entry applies to. Refer to Admin Partitions for additional information.
        """
        return pulumi.get(self, "partition")

    @partition.setter
    def partition(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "partition", value)

    @property
    @pulumi.getter
    def protocol(self) -> Optional[pulumi.Input[str]]:
        """
        Specifies the default protocol for the service.
        """
        return pulumi.get(self, "protocol")

    @protocol.setter
    def protocol(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "protocol", value)

    @property
    @pulumi.getter(name="transparentProxies")
    def transparent_proxies(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsTransparentProxyArgs']]]]:
        """
        Controls configurations specific to proxies in transparent mode. Refer to Transparent Proxy Mode for additional information.
        """
        return pulumi.get(self, "transparent_proxies")

    @transparent_proxies.setter
    def transparent_proxies(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsTransparentProxyArgs']]]]):
        pulumi.set(self, "transparent_proxies", value)

    @property
    @pulumi.getter(name="upstreamConfigs")
    def upstream_configs(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsUpstreamConfigArgs']]]]:
        """
        Controls default upstream connection settings and custom overrides for individual upstream services.
        """
        return pulumi.get(self, "upstream_configs")

    @upstream_configs.setter
    def upstream_configs(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ConfigEntryServiceDefaultsUpstreamConfigArgs']]]]):
        pulumi.set(self, "upstream_configs", value)


class ConfigEntryServiceDefaults(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 balance_inbound_connections: Optional[pulumi.Input[str]] = None,
                 destinations: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsDestinationArgs', 'ConfigEntryServiceDefaultsDestinationArgsDict']]]]] = None,
                 envoy_extensions: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsEnvoyExtensionArgs', 'ConfigEntryServiceDefaultsEnvoyExtensionArgsDict']]]]] = None,
                 exposes: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsExposeArgs', 'ConfigEntryServiceDefaultsExposeArgsDict']]]]] = None,
                 external_sni: Optional[pulumi.Input[str]] = None,
                 local_connect_timeout_ms: Optional[pulumi.Input[int]] = None,
                 local_request_timeout_ms: Optional[pulumi.Input[int]] = None,
                 max_inbound_connections: Optional[pulumi.Input[int]] = None,
                 mesh_gateways: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsMeshGatewayArgs', 'ConfigEntryServiceDefaultsMeshGatewayArgsDict']]]]] = None,
                 meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 mode: Optional[pulumi.Input[str]] = None,
                 mutual_tls_mode: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 partition: Optional[pulumi.Input[str]] = None,
                 protocol: Optional[pulumi.Input[str]] = None,
                 transparent_proxies: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsTransparentProxyArgs', 'ConfigEntryServiceDefaultsTransparentProxyArgsDict']]]]] = None,
                 upstream_configs: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsUpstreamConfigArgs', 'ConfigEntryServiceDefaultsUpstreamConfigArgsDict']]]]] = None,
                 __props__=None):
        """
        ## Example Usage

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] balance_inbound_connections: Specifies the strategy for allocating inbound connections to the service across Envoy proxy threads.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsDestinationArgs', 'ConfigEntryServiceDefaultsDestinationArgsDict']]]] destinations: Configures the destination for service traffic through terminating gateways.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsEnvoyExtensionArgs', 'ConfigEntryServiceDefaultsEnvoyExtensionArgsDict']]]] envoy_extensions: List of extensions to modify Envoy proxy configuration.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsExposeArgs', 'ConfigEntryServiceDefaultsExposeArgsDict']]]] exposes: Specifies default configurations for exposing HTTP paths through Envoy.
        :param pulumi.Input[str] external_sni: Specifies the TLS server name indication (SNI) when federating with an external system.
        :param pulumi.Input[int] local_connect_timeout_ms: Specifies the number of milliseconds allowed for establishing connections to the local application instance before timing out.
        :param pulumi.Input[int] local_request_timeout_ms: Specifies the timeout for HTTP requests to the local application instance.
        :param pulumi.Input[int] max_inbound_connections: Specifies the maximum number of concurrent inbound connections to each service instance.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsMeshGatewayArgs', 'ConfigEntryServiceDefaultsMeshGatewayArgsDict']]]] mesh_gateways: Specifies the default mesh gateway mode field for the service.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] meta: Specifies a set of custom key-value pairs to add to the Consul KV store.
        :param pulumi.Input[str] mode: Specifies a mode for how the service directs inbound and outbound traffic.
        :param pulumi.Input[str] mutual_tls_mode: Controls whether mutual TLS is required for incoming connections to this service. This setting is only supported for services with transparent proxy enabled.
        :param pulumi.Input[str] name: Specifies the name of the service you are setting the defaults for.
        :param pulumi.Input[str] namespace: Specifies the Consul namespace that the configuration entry applies to.
        :param pulumi.Input[str] partition: Specifies the name of the name of the Consul admin partition that the configuration entry applies to. Refer to Admin Partitions for additional information.
        :param pulumi.Input[str] protocol: Specifies the default protocol for the service.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsTransparentProxyArgs', 'ConfigEntryServiceDefaultsTransparentProxyArgsDict']]]] transparent_proxies: Controls configurations specific to proxies in transparent mode. Refer to Transparent Proxy Mode for additional information.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsUpstreamConfigArgs', 'ConfigEntryServiceDefaultsUpstreamConfigArgsDict']]]] upstream_configs: Controls default upstream connection settings and custom overrides for individual upstream services.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ConfigEntryServiceDefaultsArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        :param str resource_name: The name of the resource.
        :param ConfigEntryServiceDefaultsArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ConfigEntryServiceDefaultsArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 balance_inbound_connections: Optional[pulumi.Input[str]] = None,
                 destinations: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsDestinationArgs', 'ConfigEntryServiceDefaultsDestinationArgsDict']]]]] = None,
                 envoy_extensions: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsEnvoyExtensionArgs', 'ConfigEntryServiceDefaultsEnvoyExtensionArgsDict']]]]] = None,
                 exposes: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsExposeArgs', 'ConfigEntryServiceDefaultsExposeArgsDict']]]]] = None,
                 external_sni: Optional[pulumi.Input[str]] = None,
                 local_connect_timeout_ms: Optional[pulumi.Input[int]] = None,
                 local_request_timeout_ms: Optional[pulumi.Input[int]] = None,
                 max_inbound_connections: Optional[pulumi.Input[int]] = None,
                 mesh_gateways: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsMeshGatewayArgs', 'ConfigEntryServiceDefaultsMeshGatewayArgsDict']]]]] = None,
                 meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 mode: Optional[pulumi.Input[str]] = None,
                 mutual_tls_mode: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 namespace: Optional[pulumi.Input[str]] = None,
                 partition: Optional[pulumi.Input[str]] = None,
                 protocol: Optional[pulumi.Input[str]] = None,
                 transparent_proxies: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsTransparentProxyArgs', 'ConfigEntryServiceDefaultsTransparentProxyArgsDict']]]]] = None,
                 upstream_configs: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsUpstreamConfigArgs', 'ConfigEntryServiceDefaultsUpstreamConfigArgsDict']]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ConfigEntryServiceDefaultsArgs.__new__(ConfigEntryServiceDefaultsArgs)

            __props__.__dict__["balance_inbound_connections"] = balance_inbound_connections
            __props__.__dict__["destinations"] = destinations
            __props__.__dict__["envoy_extensions"] = envoy_extensions
            if exposes is None and not opts.urn:
                raise TypeError("Missing required property 'exposes'")
            __props__.__dict__["exposes"] = exposes
            __props__.__dict__["external_sni"] = external_sni
            __props__.__dict__["local_connect_timeout_ms"] = local_connect_timeout_ms
            __props__.__dict__["local_request_timeout_ms"] = local_request_timeout_ms
            __props__.__dict__["max_inbound_connections"] = max_inbound_connections
            __props__.__dict__["mesh_gateways"] = mesh_gateways
            __props__.__dict__["meta"] = meta
            __props__.__dict__["mode"] = mode
            __props__.__dict__["mutual_tls_mode"] = mutual_tls_mode
            __props__.__dict__["name"] = name
            __props__.__dict__["namespace"] = namespace
            __props__.__dict__["partition"] = partition
            if protocol is None and not opts.urn:
                raise TypeError("Missing required property 'protocol'")
            __props__.__dict__["protocol"] = protocol
            __props__.__dict__["transparent_proxies"] = transparent_proxies
            __props__.__dict__["upstream_configs"] = upstream_configs
        super(ConfigEntryServiceDefaults, __self__).__init__(
            'consul:index/configEntryServiceDefaults:ConfigEntryServiceDefaults',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            balance_inbound_connections: Optional[pulumi.Input[str]] = None,
            destinations: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsDestinationArgs', 'ConfigEntryServiceDefaultsDestinationArgsDict']]]]] = None,
            envoy_extensions: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsEnvoyExtensionArgs', 'ConfigEntryServiceDefaultsEnvoyExtensionArgsDict']]]]] = None,
            exposes: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsExposeArgs', 'ConfigEntryServiceDefaultsExposeArgsDict']]]]] = None,
            external_sni: Optional[pulumi.Input[str]] = None,
            local_connect_timeout_ms: Optional[pulumi.Input[int]] = None,
            local_request_timeout_ms: Optional[pulumi.Input[int]] = None,
            max_inbound_connections: Optional[pulumi.Input[int]] = None,
            mesh_gateways: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsMeshGatewayArgs', 'ConfigEntryServiceDefaultsMeshGatewayArgsDict']]]]] = None,
            meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            mode: Optional[pulumi.Input[str]] = None,
            mutual_tls_mode: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            namespace: Optional[pulumi.Input[str]] = None,
            partition: Optional[pulumi.Input[str]] = None,
            protocol: Optional[pulumi.Input[str]] = None,
            transparent_proxies: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsTransparentProxyArgs', 'ConfigEntryServiceDefaultsTransparentProxyArgsDict']]]]] = None,
            upstream_configs: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsUpstreamConfigArgs', 'ConfigEntryServiceDefaultsUpstreamConfigArgsDict']]]]] = None) -> 'ConfigEntryServiceDefaults':
        """
        Get an existing ConfigEntryServiceDefaults resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] balance_inbound_connections: Specifies the strategy for allocating inbound connections to the service across Envoy proxy threads.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsDestinationArgs', 'ConfigEntryServiceDefaultsDestinationArgsDict']]]] destinations: Configures the destination for service traffic through terminating gateways.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsEnvoyExtensionArgs', 'ConfigEntryServiceDefaultsEnvoyExtensionArgsDict']]]] envoy_extensions: List of extensions to modify Envoy proxy configuration.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsExposeArgs', 'ConfigEntryServiceDefaultsExposeArgsDict']]]] exposes: Specifies default configurations for exposing HTTP paths through Envoy.
        :param pulumi.Input[str] external_sni: Specifies the TLS server name indication (SNI) when federating with an external system.
        :param pulumi.Input[int] local_connect_timeout_ms: Specifies the number of milliseconds allowed for establishing connections to the local application instance before timing out.
        :param pulumi.Input[int] local_request_timeout_ms: Specifies the timeout for HTTP requests to the local application instance.
        :param pulumi.Input[int] max_inbound_connections: Specifies the maximum number of concurrent inbound connections to each service instance.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsMeshGatewayArgs', 'ConfigEntryServiceDefaultsMeshGatewayArgsDict']]]] mesh_gateways: Specifies the default mesh gateway mode field for the service.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] meta: Specifies a set of custom key-value pairs to add to the Consul KV store.
        :param pulumi.Input[str] mode: Specifies a mode for how the service directs inbound and outbound traffic.
        :param pulumi.Input[str] mutual_tls_mode: Controls whether mutual TLS is required for incoming connections to this service. This setting is only supported for services with transparent proxy enabled.
        :param pulumi.Input[str] name: Specifies the name of the service you are setting the defaults for.
        :param pulumi.Input[str] namespace: Specifies the Consul namespace that the configuration entry applies to.
        :param pulumi.Input[str] partition: Specifies the name of the name of the Consul admin partition that the configuration entry applies to. Refer to Admin Partitions for additional information.
        :param pulumi.Input[str] protocol: Specifies the default protocol for the service.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsTransparentProxyArgs', 'ConfigEntryServiceDefaultsTransparentProxyArgsDict']]]] transparent_proxies: Controls configurations specific to proxies in transparent mode. Refer to Transparent Proxy Mode for additional information.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ConfigEntryServiceDefaultsUpstreamConfigArgs', 'ConfigEntryServiceDefaultsUpstreamConfigArgsDict']]]] upstream_configs: Controls default upstream connection settings and custom overrides for individual upstream services.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ConfigEntryServiceDefaultsState.__new__(_ConfigEntryServiceDefaultsState)

        __props__.__dict__["balance_inbound_connections"] = balance_inbound_connections
        __props__.__dict__["destinations"] = destinations
        __props__.__dict__["envoy_extensions"] = envoy_extensions
        __props__.__dict__["exposes"] = exposes
        __props__.__dict__["external_sni"] = external_sni
        __props__.__dict__["local_connect_timeout_ms"] = local_connect_timeout_ms
        __props__.__dict__["local_request_timeout_ms"] = local_request_timeout_ms
        __props__.__dict__["max_inbound_connections"] = max_inbound_connections
        __props__.__dict__["mesh_gateways"] = mesh_gateways
        __props__.__dict__["meta"] = meta
        __props__.__dict__["mode"] = mode
        __props__.__dict__["mutual_tls_mode"] = mutual_tls_mode
        __props__.__dict__["name"] = name
        __props__.__dict__["namespace"] = namespace
        __props__.__dict__["partition"] = partition
        __props__.__dict__["protocol"] = protocol
        __props__.__dict__["transparent_proxies"] = transparent_proxies
        __props__.__dict__["upstream_configs"] = upstream_configs
        return ConfigEntryServiceDefaults(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="balanceInboundConnections")
    def balance_inbound_connections(self) -> pulumi.Output[Optional[str]]:
        """
        Specifies the strategy for allocating inbound connections to the service across Envoy proxy threads.
        """
        return pulumi.get(self, "balance_inbound_connections")

    @property
    @pulumi.getter
    def destinations(self) -> pulumi.Output[Optional[Sequence['outputs.ConfigEntryServiceDefaultsDestination']]]:
        """
        Configures the destination for service traffic through terminating gateways.
        """
        return pulumi.get(self, "destinations")

    @property
    @pulumi.getter(name="envoyExtensions")
    def envoy_extensions(self) -> pulumi.Output[Optional[Sequence['outputs.ConfigEntryServiceDefaultsEnvoyExtension']]]:
        """
        List of extensions to modify Envoy proxy configuration.
        """
        return pulumi.get(self, "envoy_extensions")

    @property
    @pulumi.getter
    def exposes(self) -> pulumi.Output[Sequence['outputs.ConfigEntryServiceDefaultsExpose']]:
        """
        Specifies default configurations for exposing HTTP paths through Envoy.
        """
        return pulumi.get(self, "exposes")

    @property
    @pulumi.getter(name="externalSni")
    def external_sni(self) -> pulumi.Output[Optional[str]]:
        """
        Specifies the TLS server name indication (SNI) when federating with an external system.
        """
        return pulumi.get(self, "external_sni")

    @property
    @pulumi.getter(name="localConnectTimeoutMs")
    def local_connect_timeout_ms(self) -> pulumi.Output[Optional[int]]:
        """
        Specifies the number of milliseconds allowed for establishing connections to the local application instance before timing out.
        """
        return pulumi.get(self, "local_connect_timeout_ms")

    @property
    @pulumi.getter(name="localRequestTimeoutMs")
    def local_request_timeout_ms(self) -> pulumi.Output[Optional[int]]:
        """
        Specifies the timeout for HTTP requests to the local application instance.
        """
        return pulumi.get(self, "local_request_timeout_ms")

    @property
    @pulumi.getter(name="maxInboundConnections")
    def max_inbound_connections(self) -> pulumi.Output[Optional[int]]:
        """
        Specifies the maximum number of concurrent inbound connections to each service instance.
        """
        return pulumi.get(self, "max_inbound_connections")

    @property
    @pulumi.getter(name="meshGateways")
    def mesh_gateways(self) -> pulumi.Output[Optional[Sequence['outputs.ConfigEntryServiceDefaultsMeshGateway']]]:
        """
        Specifies the default mesh gateway mode field for the service.
        """
        return pulumi.get(self, "mesh_gateways")

    @property
    @pulumi.getter
    def meta(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        Specifies a set of custom key-value pairs to add to the Consul KV store.
        """
        return pulumi.get(self, "meta")

    @property
    @pulumi.getter
    def mode(self) -> pulumi.Output[Optional[str]]:
        """
        Specifies a mode for how the service directs inbound and outbound traffic.
        """
        return pulumi.get(self, "mode")

    @property
    @pulumi.getter(name="mutualTlsMode")
    def mutual_tls_mode(self) -> pulumi.Output[Optional[str]]:
        """
        Controls whether mutual TLS is required for incoming connections to this service. This setting is only supported for services with transparent proxy enabled.
        """
        return pulumi.get(self, "mutual_tls_mode")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        Specifies the name of the service you are setting the defaults for.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def namespace(self) -> pulumi.Output[Optional[str]]:
        """
        Specifies the Consul namespace that the configuration entry applies to.
        """
        return pulumi.get(self, "namespace")

    @property
    @pulumi.getter
    def partition(self) -> pulumi.Output[Optional[str]]:
        """
        Specifies the name of the name of the Consul admin partition that the configuration entry applies to. Refer to Admin Partitions for additional information.
        """
        return pulumi.get(self, "partition")

    @property
    @pulumi.getter
    def protocol(self) -> pulumi.Output[str]:
        """
        Specifies the default protocol for the service.
        """
        return pulumi.get(self, "protocol")

    @property
    @pulumi.getter(name="transparentProxies")
    def transparent_proxies(self) -> pulumi.Output[Optional[Sequence['outputs.ConfigEntryServiceDefaultsTransparentProxy']]]:
        """
        Controls configurations specific to proxies in transparent mode. Refer to Transparent Proxy Mode for additional information.
        """
        return pulumi.get(self, "transparent_proxies")

    @property
    @pulumi.getter(name="upstreamConfigs")
    def upstream_configs(self) -> pulumi.Output[Optional[Sequence['outputs.ConfigEntryServiceDefaultsUpstreamConfig']]]:
        """
        Controls default upstream connection settings and custom overrides for individual upstream services.
        """
        return pulumi.get(self, "upstream_configs")

